@@ -6,11 +6,12 @@ import time
 import threading
 import pkg_resources
 import json
+import functools
 from velbus.parser import VelbusParser
 from velbus.connections.socket import SocketConnection
 from velbus.connections.serial import VelbusUSBConnection
-from velbus.messages.module_type_request import ModuleTypeRequestMessage
 from velbus.message import Message
+from velbus.messages.module_type_request import ModuleTypeRequestMessage
 from velbus.messages.bus_active import BusActiveMessage
 from velbus.messages.receive_ready import ReceiveReadyMessage
 from velbus.messages.bus_off import BusOffMessage
@@ -31,7 +32,8 @@ class Controller(object):
     def __init__(self, port):
         self.logger = logging.getLogger("velbus")
         self.parser = VelbusParser(self)
-        self.__subscribers = []
+        self.__message_subscribers = []
+        self.__module_subscribers = {}
         self.__scan_callback = None
         self._modules = {}
         self._loadModuleData()
@@ -40,6 +42,13 @@ class Controller(object):
         else:
             self.connection = VelbusUSBConnection(port, self)
 
+    def _loadModuleData(self):
+        filepath = pkg_resources.resource_filename(__name__, "data.json")
+        with open(filepath) as json_file:
+            self._module_data = json.load(json_file)
+
+    # interface towards connection objects
+
     def feed_parser(self, data):
         """
         Feed parser with new data
@@ -49,23 +58,35 @@ class Controller(object):
         assert isinstance(data, bytes)
         self.parser.feed(data)
 
+    # event interface
+
     def subscribe(self, subscriber):
         """
         :return: None
         """
-        self.__subscribers.append(subscriber)
+        self.__message_subscribers.append(subscriber)
 
-    def parse(self, binary_message):
+    def subscribe_module(self, subscriber, category):
         """
-        :return: velbus.Message or None
+        :return: None
         """
-        return self.parser.parse(binary_message)
+        if category not in self.__module_subscribers:
+            self.__module_subscribers[category] = []
+        self.__module_subscribers[category].append(subscriber)
 
     def unsubscribe(self, subscriber):
         """
         :return: None
         """
-        self.__subscribers.remove(subscriber)
+        self.__message_subscribers.remove(subscriber)
+
+    def unsubscribe_module(self, subscriber, category):
+        """
+        :return: None
+        """
+        self.__module_subscribers[category].remove(subscriber)
+
+    # command interface
 
     def send(self, message, callback=None):
         """
@@ -75,12 +96,24 @@ class Controller(object):
 
     def get_modules(self):
         """
-        Returns a list of modules from a specific category
+        Returns a list of modules
 
         :return: list
         """
         return self._modules.values()
 
+    def get_modules_loaded(self):
+        """
+        Returns a list of loaded modules
+
+        :return: list
+        """
+        result = []
+        for module in self.get_modules():
+            if module.loaded:
+                result.append(module)
+        return result
+
     def get_module(self, address):
         """
         Returns module at address
@@ -144,91 +177,10 @@ class Controller(object):
             else:
                 self.send(message)
 
-    def send_binary(self, binary_message, callback=None):
-        """
-        :return: None
-        """
-        assert isinstance(binary_message, str)
-        message = self.parser.parse(binary_message)
-        if isinstance(message, Message):
-            self.send(message, callback)
-
-    def new_message(self, message):
-        """
-        :return: None
-        """
-        self.logger.info("New message: " + str(message))
-        if isinstance(message, ModuleTypeMessage):
-            self.logger.debug(
-                "Module type response received from address " + str(message.address)
-            )
-            name = message.module_name()
-            address = message.address
-            m_type = message.module_type
-            if name == "Unknown":
-                self.logger.warning(
-                    "Unknown module (code: " + str(message.module_type) + ")"
-                )
-                return
-            if name in ModuleRegistry:
-                module = ModuleRegistry[name](m_type, name, address, self)
-                self._modules[address] = module
-            else:
-                self.logger.warning("Module " + name + " is not yet supported")
-        elif isinstance(message, ModuleSubTypeMessage):
-            self.logger.debug(
-                "Module subtype response received from address " + str(message.address)
-            )
-            name = message.module_name()
-            address = message.address
-            m_type = message.module_type
-            if name == "Unknown":
-                self.logger.warning(
-                    "Unknown module (code: " + str(message.module_type) + ")"
-                )
-                return
-            if "SUB_" + name in ModuleRegistry:
-                subname = "SUB_" + name
-                if message.sub_address_1 != 0xFF:
-                    module = ModuleRegistry[subname](
-                        m_type, subname, message.sub_address_1, address, 1, self,
-                    )
-                    self._modules[message.sub_address_1] = module
-                if message.sub_address_2 != 0xFF:
-                    module = ModuleRegistry[subname](
-                        m_type, subname, message.sub_address_2, address, 2, self,
-                    )
-                    self._modules[message.sub_address_2] = module
-                if message.sub_address_3 != 0xFF:
-                    module = ModuleRegistry[subname](
-                        m_type, subname, message.sub_address_3, address, 3, self,
-                    )
-                    self._modules[message.sub_address_3] = module
-                if (
-                    message.sub_address_4 != 0xFF
-                    and name != "VMBGPOD"
-                    and name != "VMBGPO"
-                    and name != "VMBELO"
-                ):
-                    module = ModuleRegistry[subname](
-                        m_type, subname, message.sub_address_4, address, 4, self,
-                    )
-                    self._modules[message.sub_address_4] = module
-            else:
-                self.logger.warning(
-                    "Module " + name + " does not yet support sub modules"
-                )
-        elif isinstance(message, BusActiveMessage):
-            self.logger.info("Velbus active message received")
-        elif isinstance(message, ReceiveReadyMessage):
-            self.logger.info("Velbus receive ready message received")
-        elif isinstance(message, BusOffMessage):
-            self.logger.error("Velbus bus off message received")
-        elif isinstance(message, ReceiveBufferFullMessage):
-            self.logger.error("Velbus receive buffer full message received")
-        # notify everyone who requests it
-        for subscriber in self.__subscribers:
-            subscriber(message)
+    def async_scan(self):
+        for address in range(0, 256):
+            message = ModuleTypeRequestMessage(address)
+            self.send(message)
 
     def stop(self):
         """
@@ -244,7 +196,157 @@ class Controller(object):
         self.send(SetDate())
         self.send(SetDaylightSaving())
 
-    def _loadModuleData(self):
-        filepath = pkg_resources.resource_filename(__name__, "data.json")
-        with open(filepath) as json_file:
-            self._module_data = json.load(json_file)
+    # messaging and module loading logic
+
+    def new_message(self, message):
+        """
+        :return: None
+        """
+        self.logger.info("New message: " + str(message))
+        if isinstance(message, ModuleTypeMessage):
+            self._process_module_type_message(message)
+        elif isinstance(message, ModuleSubTypeMessage):
+            self._process_module_subtype_message(message)
+        elif isinstance(message, BusActiveMessage):
+            self.logger.info("Velbus active message received")
+        elif isinstance(message, ReceiveReadyMessage):
+            self.logger.info("Velbus receive ready message received")
+        elif isinstance(message, BusOffMessage):
+            self.logger.error("Velbus bus off message received")
+        elif isinstance(message, ReceiveBufferFullMessage):
+            self.logger.error("Velbus receive buffer full message received")
+        # notify everyone who requests it
+        for subscriber in self.__message_subscribers:
+            subscriber(message)
+
+    def _process_module_type_message(self, message):
+        """
+        Process ModuleType message and if new module: add to module repository
+        """
+        self.logger.debug(
+            "Module type response received from address " + str(message.address)
+        )
+        name = message.module_name()
+        address = message.address
+        m_type = message.module_type
+        if name == "Unknown":
+            self.logger.warning(
+                "Unknown module (code: " + str(message.module_type) + ")"
+            )
+            return
+        if name in ModuleRegistry:
+            module = ModuleRegistry[name](m_type, name, address, self)
+            self._add_module(address, module)
+        else:
+            self.logger.warning("Module " + name + " is not yet supported")
+
+    def _process_module_subtype_message(self, message):
+        """
+        Process ModuleSubType message and if new module: add to module repository
+        """
+        self.logger.debug(
+            "Module subtype response received from address " + str(message.address)
+        )
+        name = message.module_name()
+        address = message.address
+        m_type = message.module_type
+        if name == "Unknown":
+            self.logger.warning(
+                "Unknown module (code: " + str(message.module_type) + ")"
+            )
+            return
+        if "SUB_" + name in ModuleRegistry:
+            subname = "SUB_" + name
+            if message.sub_address_1 != 0xFF:
+                module = ModuleRegistry[subname](
+                    m_type,
+                    subname,
+                    message.sub_address_1,
+                    address,
+                    1,
+                    self,
+                )
+                self._add_module(message.sub_address_1, module)
+            if message.sub_address_2 != 0xFF:
+                module = ModuleRegistry[subname](
+                    m_type,
+                    subname,
+                    message.sub_address_2,
+                    address,
+                    2,
+                    self,
+                )
+                self._add_module(message.sub_address_2, module)
+            if message.sub_address_3 != 0xFF:
+                module = ModuleRegistry[subname](
+                    m_type,
+                    subname,
+                    message.sub_address_3,
+                    address,
+                    3,
+                    self,
+                )
+                self._add_module(message.sub_address_3, module)
+            if (
+                message.sub_address_4 != 0xFF
+                and name != "VMBGPOD"
+                and name != "VMBGPO"
+                and name != "VMBELO"
+            ):
+                module = ModuleRegistry[subname](
+                    m_type,
+                    subname,
+                    message.sub_address_4,
+                    address,
+                    4,
+                    self,
+                )
+                self._add_module(message.sub_address_4, module)
+        else:
+            self.logger.warning("Module " + name + " does not yet support sub modules")
+
+    def _add_module(self, address, module):
+        callback = functools.partial(self._module_loaded, module)
+        if address not in self._modules:
+            self.logger.info("adding module at address %s", address)
+            self._modules[address] = module
+            module.load(callback)
+        else:
+            if self._modules[address].loaded:
+                self.logger.info(
+                    "module already in registry at address %s and loaded", address
+                )
+            else:
+                self.logger.info("loading module at address %s", address)
+                module.load(callback)
+
+    def _module_loaded(self, module):
+        for channel in range(1, module.number_of_channels() + 1):
+            categories = module.get_categories(channel)
+            for category in categories:
+                if category in self.__module_subscribers:
+                    for subscriber in self.__module_subscribers[category]:
+                        subscriber(module, channel)
+
+    # probably not used
+
+    def parse(self, binary_message):
+        """
+        :return: velbus.Message or None
+        """
+        return self.parser.parse(binary_message)
+
+    def send_binary(self, binary_message, callback=None):
+        """
+        :return: None
+        """
+        assert isinstance(binary_message, str)
+        message = self.parser.parse(binary_message)
+        if isinstance(message, Message):
+            self.send(message, callback)
+
+    def new_binary_message(self, message):
+        assert isinstance(message, bytes)
+        message = self.parser.parse_binary_message(message)
+        if isinstance(message, Message):
+            self.new_message(message)
